use std::{cmp::Ordering::Greater, fmt::{Debug, Display}, ops::{Index, IndexMut}};
use point::Point;
use serde::{Deserialize, Serialize};

fn point_to_index(p: &Point<u32>) -> usize {
    ((p.y * 32) + p.x) as usize
}

#[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
pub struct Chunk {
    tiles: Vec<u64>
}

impl Index<Point<u32>> for Chunk {
    type Output = u64;
    fn index(&self, index: Point<u32>) -> &Self::Output {
        let idx = point_to_index(&index);
        if idx >= 32*32 {
            panic!("index out of bounds");
        }
        &self.tiles[idx]
    }
}

impl IndexMut<Point<u32>> for Chunk {
    fn index_mut(&mut self, index: Point<u32>) -> &mut Self::Output {
        let idx = point_to_index(&index);
        if idx >= 32*32 {
            panic!("index out of bounds");
        }
        &mut self.tiles[idx]
    }
}

impl Index<&Point<u32>> for Chunk {
    type Output = u64;
    fn index(&self, index: &Point<u32>) -> &Self::Output {
        let idx = point_to_index(index);
        if idx >= 32*32 {
            panic!("index out of bounds");
        }
        &self.tiles[idx]
    }
}

impl IndexMut<&Point<u32>> for Chunk {
    fn index_mut(&mut self, index: &Point<u32>) -> &mut Self::Output {
        let idx = point_to_index(index);
        if idx >= 32*32 {
            panic!("index out of bounds");
        }
        &mut self.tiles[idx]
    }
}

impl Chunk {
    pub fn new() -> Self {
        let mut s = Self { tiles: Vec::with_capacity(1024) };
        s.tiles.extend_from_slice(&[0u64; 1024]);
        s
    }
    pub fn new_filled(value: u64) -> Self {
        let mut s = Self { tiles: Vec::with_capacity(1024) };
        s.tiles.extend_from_slice(&[value; 1024]);
        s
    }
    pub fn get(&self, pos: &Point<u32>) -> Option<&u64> {
        let idx = point_to_index(pos);
        if idx >= 32*32 {
            return None;
        }
        self.tiles.get(idx)
    }
    pub fn get_mut(&mut self, pos: &Point<u32>) -> Option<&mut u64> {
        let idx = point_to_index(pos);
        if idx >= 32*32 {
            return None;
        }
        self.tiles.get_mut(idx)
    }
    pub fn set(&mut self, pos: &Point<u32>, value: u64) -> bool {
        let idx = point_to_index(pos);
        if idx >= 32*32 {
            return false;
        }
        self.tiles[idx] = value;
        true
    }
    pub fn fill(&mut self, pos1: &Point<u32>, pos2: &Point<u32>, value: u64) -> bool {
        let (corn1, corn2) = match pos1.partial_cmp(pos2).unwrap() {
            Greater => {
                (point_to_index(pos2), point_to_index(pos1))
            },
            _ => {
                (point_to_index(pos1), point_to_index(pos2))
            }
        };
        if corn1 >= 32*32 || corn2 >= 32*32 {
            return false;
        }
        for idx in corn1..corn2 {
            self.tiles[idx as usize] = value;
        }
        true
    }
    pub fn fill_all(&mut self, value: u64) {
        self.tiles.clear();
        self.tiles.extend_from_slice(&[value; 1024]);
    }
    pub fn clear(&mut self) {
        self.tiles.clear();
        self.tiles.extend_from_slice(&[0u64; 1024]);
    }
}

impl Default for Chunk {
    fn default() -> Self {
        Self::new()
    }
}

impl Display for Chunk {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        for y in 0..32u64 {
            for x in 0..32u64 {
                let idx = (x + y) as usize;
                f.write_str(format!("{}", self.tiles[idx]).as_str())?;
            }
            f.write_str("\r\n")?;
        }
        Ok(())
    }
}
