use std::{collections::HashMap, sync::Arc, time};

use anyhow::Ok;
use async_trait::async_trait;
use ratatui::{
    prelude::CrosstermBackend, style::{Color, Style}, widgets::{Block, Borders, Paragraph}, Terminal
};
use russh::{server::{Msg, Session}, Channel};
use shared::Shared;
use crate::{sshterm::{SshBackend, TerminalHandle}, SshTerminal};

#[derive(Clone)]
pub struct AppServer {
    pub id: usize,
    pub clients: Shared<HashMap<usize, SshTerminal>>
}

impl AppServer {
    pub fn new() -> Self {
        Self {
            id: 0,
            clients: Shared::new(HashMap::new())
        }
    }
    pub async fn run(&mut self, address: (&str, u32)) -> Result<(), anyhow::Error> {
        let clients = self.clients.clone();
        tokio::spawn(async move {
            loop {
                for (_, t) in clients.lock().await.iter_mut() {
                    t.draw(|f| {
                        let b = Block::new()
                            .borders(Borders::ALL)
                            .title("title");
                        let p = Paragraph::new("Hello, World!")
                            .alignment(ratatui::layout::Alignment::Center)
                            .style(Style::default());
                        f.render_widget(p.block(b), f.area());
                    })?;
                }
            }
        });
        let config = russh::server::Config{
            inactivity_timeout: Some(time::Duration::from_secs(3600)),
            auth_rejection_time: time::Duration::from_secs(3),
            auth_rejection_time_initial: Some(time::Duration::from_secs(0)),
            keys: vec![russh_keys::key::KeyPair::generate_ed25519().unwrap()],
            ..Default::default()
        };
        self.run_on_address(Arc::new(config), address);
        Ok(())
    }
}

impl russh::server::Server for AppServer {
    type Handler = Self;
    fn new_client(&mut self,peer_addr:Option<std::net::SocketAddr>) -> Self::Handler {
        let s = self.clone();
        self.id += 1;
        s
    }
}

#[async_trait]
impl russh::server::Handler for AppServer {
    type Error = anyhow::Error;
    async fn channel_open_session(&mut self, channel: Channel<Msg>, session: &mut Session) -> Result<bool, Self::Error> {
        {
            let mut clients = self.clients.lock().await;
            let backend = SshBackend::new(channel.id(), session.handle(), 0, 0, 0, 0);
            let term = SshTerminal::new(backend)?;
            clients.insert(self.id, term);
        }
        Ok(true)
    }
}
