use std::{fs, path::PathBuf, iter::Iterator};

use serde::{Deserialize, Serialize};

use crate::Tile;

#[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
pub struct TileIndex {
    tiles: Vec<Tile>
}

impl TileIndex {
    pub fn new() -> Self {
        Self { tiles: Vec::new() }
    }
    pub fn load(path: PathBuf) -> Result<Self, anyhow::Error> {
        let content = fs::read_to_string(path)?;
        let index = serde_json::from_str::<TileIndex>(&content)?;
        Ok(index)
    }
    pub fn save(&self, path: PathBuf) -> Result<(), anyhow::Error> {
        let content = serde_json::to_string_pretty(self)?;
        let p = path.to_string_lossy().to_string();
        let p = p.strip_suffix(path.file_name().unwrap().to_string_lossy().to_string().as_str()).unwrap();
        let p  = PathBuf::from(p);
        if !p.exists() {
            fs::create_dir_all(p)?;
        }
        fs::write(path, content)?;
        Ok(())
    }
    pub fn len(&self) -> usize {
        self.tiles.len()
    }
    pub fn lookup_index(&self, name: &str) -> Option<u64> {
        let mut idx: u64 = 0;
        for t in self.tiles.iter() {
            if t.name == name.to_string() {
                return Some(idx);
            }
            idx += 1;
        }
        None
    }
    pub fn get_name(&self, name: &str) -> Option<&Tile> {
        for t in self.tiles.iter() {
            if t.name == name.to_string() {
                return Some(t);
            }
        }
        None
    }
    pub fn get_index(&self, index: u64) -> Option<&Tile> {
        if index >= self.tiles.len() as u64 {
            return None;
        }
        Some(&self.tiles[index as usize])
    }
    pub fn set(&mut self, t: Tile) -> usize {
        if let Some(idx) = self.lookup_index(&t.name) {
            self.tiles[idx as usize] = t;
            return idx as usize;
        }
        self.tiles.push(t);
        self.tiles.len()
    }
    pub fn define(&mut self, t: Tile) -> bool {
        if let Some(_idx) = self.lookup_index(&t.name) {
            return false;
        }
        self.tiles.push(t);
        true
    }
}

impl Default for TileIndex {
    fn default() -> Self {
        Self::new()
    }
}

pub struct TileIndexIter {
    tile_index: TileIndex,
    index: u64,
}

impl TileIndexIter {
    pub fn new(index: TileIndex) -> Self {
        Self { tile_index: index, index: 0 }
    }
}

impl Iterator for TileIndexIter {
    type Item = Tile;
    fn next(&mut self) -> Option<Self::Item> {
        self.index += 1;
        if let Some(t) = self.tile_index.get_index(self.index-1) {
            Some(t.clone())
        } else {
            None
        }
    }
}

impl IntoIterator for TileIndex {
    type IntoIter = TileIndexIter;
    type Item = Tile;
    fn into_iter(self) -> Self::IntoIter {
        TileIndexIter::new(self)
    }
}
